/* ------ PARLNK.H ------
written by Flavien RAMEAU (rameau.fl@lemel.fr)
uses code from Titerm.c by Bryan Rittmeyer

There is 4 function :
	init_plink : it initialize the parrallel port
	chk_plink  : it test the $5 parrallel link
	get_plink  : it read 8 bit from the calculator
	put_plink  : it send 8 bit to the calculator

*/


#define lpt_out 0x378
#define lpt_in (lpt_out + 1)
#define MaxTimeAns 10000

int init_plink(void);
int chk_plink(void);
int get_plink(void);
int put_plink(unsigned char data);

int init_plink(void)
{
	outportb(lpt_out, 3);
	return 0;
}

int chk_plink(void)
{
	char r;

	outportb(lpt_out,1);
	delay(1);
	r = inportb(lpt_in);
	if (!( (r & 0x30) == 0x20))
	{
		outportb(lpt_out, 3);
		return -1;
	}

	outportb(lpt_out,2);
	delay(1);
	r = inportb(lpt_in);
	if (!( (r & 0x30) == 0x10))
	{
		outport(lpt_out,3);
		return -1;
	}

	outportb(lpt_out,3);
	delay(1);
	r = inportb(lpt_in);
	if (!( (r & 0x30) == 0x30)) return -1;
	return 0;
}

int get_plink(void)
{
	int bit, TiTime = 0;
	unsigned char data = 0;
	unsigned char v;

	for (bit = 0; bit < 8; bit++)
	{
		while ((v=inportb(lpt_in) & 0x30) == 0x30)
		{
			delay(100);
			if (TiTime++ > MaxTimeAns) return -1;
		}
		if (v == 0x10)
		{
			data = (data >>1) | 0x80;
			outportb(lpt_out, 1);
			while ((inportb(lpt_in) & 0x20) == 0x00) if (TiTime++ > MaxTimeAns) return -1;
			outportb(lpt_out, 3);
		}
		else
		{
			data = data >>1;
			outportb(lpt_out, 2);
			while ((inportb(lpt_in) & 0x10) == 0x00) if (TiTime++ > MaxTimeAns) return -1;
			outportb(lpt_out, 3);
		}
		inportb(lpt_in);
		inportb(lpt_in);
		inportb(lpt_in);
	}
	return data;
}

int put_plink(unsigned char data)
{
	int bit, TiTime = 0;

	for(bit=0; bit < 8; bit++)
	{
		inportb(lpt_in);
		inportb(lpt_in);
		inportb(lpt_in);
		if (data & 1)
		{
			outportb(lpt_out, 2);
			while (inportb(lpt_in) & 0x10) if (TiTime > MaxTimeAns) return -1;
			outportb(lpt_out, 3);
			while ((inportb(lpt_in) & 0x10) == 0x00) if (TiTime > MaxTimeAns) return -1;
		}
		else
		{
			outportb(lpt_out, 1);
			while (inportb(lpt_in) & 0x20) if (TiTime > MaxTimeAns) return -1;
			outportb(lpt_out, 3);
			while ((inportb(lpt_in) & 0x20) == 0x00) if (TiTime > MaxTimeAns) return -1;
		}
		data >>=1;
	}
	return 0;
}